package schedframe.resources.computing.profiles.load;

import java.util.Properties;

import schedframe.events.Event;
import schedframe.resources.computing.extensions.Extension;
import schedframe.resources.computing.extensions.ExtensionException;
import schedframe.resources.computing.extensions.ExtensionType;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.profiles.energy.EnergyEventType;
import schedframe.resources.computing.profiles.load.ui.LoadInterface;
import schedframe.resources.devices.PhysicalResource;
import schedframe.scheduling.manager.tasks.JobRegistryImpl;

public class LoadExtension implements Extension {

	protected LoadInterface loadInterface;
	protected LoadProfile loadProfile;
  	protected PhysicalResource resource;

	public LoadExtension(LoadProfile loadProfile,  PhysicalResource resource) {
		this.loadProfile = loadProfile;
		this.loadInterface =  LoadInterfaceFactory.createLoadInterface(resource, loadProfile);
		this.resource = resource;
	}

	public boolean supportsEvent(Event event) {

		if(loadProfile == null || loadProfile.getLoadEstimationPlugin() == null)
			return false;
		if(event.getType().getName().equals(EnergyEventType.POWER_STATE_CHANGED.getName()))
			return true;
		else if(event.getType().getName().equals(EnergyEventType.FREQUENCY_CHANGED.getName()))
			return true;
		else if(event.getType().getName().equals(EnergyEventType.TASK_STARTED.getName()))
			return true;
		else if(event.getType().getName().equals(EnergyEventType.TASK_FINISHED.getName()))
			return true;
		else if(event.getType().getName().equals(EnergyEventType.RESOURCE_UTILIZATION_CHANGED.getName()))
			return true;		
		else return false;
	}

	public void handleEvent(Event event) {
		
		EnergyEvent enEvent = (EnergyEvent)event;
		double load = 0;
		try{
			switch (enEvent.getType()) {
			
			case POWER_STATE_CHANGED:
				load = loadProfile.getLoadEstimationPlugin().estimateUtlization(enEvent, new JobRegistryImpl(resource.getFullName()), resource);
				loadProfile.addToLoadHistory(load);
				break;
				
			case FREQUENCY_CHANGED:
				load = loadProfile.getLoadEstimationPlugin().estimateUtlization(enEvent, new JobRegistryImpl(resource.getFullName()), resource);
				loadProfile.addToLoadHistory(load);
				break;
				
			case TASK_STARTED:
				load = loadProfile.getLoadEstimationPlugin().estimateUtlization(enEvent, new JobRegistryImpl(resource.getFullName()), resource);
				loadProfile.addToLoadHistory(load);
				break;
	
			case TASK_FINISHED:
				load = loadProfile.getLoadEstimationPlugin().estimateUtlization(enEvent, new JobRegistryImpl(resource.getFullName()), resource);
				loadProfile.addToLoadHistory(load);
				break;
				
			case RESOURCE_UTILIZATION_CHANGED:
				load = loadProfile.getLoadEstimationPlugin().estimateUtlization(enEvent, new JobRegistryImpl(resource.getFullName()), resource);
				loadProfile.addToLoadHistory(load);
				break;

			}
		}catch(Exception e){
			
		}
	}

	@Override
	public void init(Properties properties) throws ExtensionException {
		// TODO Auto-generated method stub
		
	}

	@Override
	public ExtensionType getType() {
		return ExtensionType.LOAD_EXTENSION;
	}

	public LoadProfile getLoadProfile() {
		return loadProfile;
	}

	public LoadInterface getLoadInterface() {
		return loadInterface;
	}

}
